//+------------------------------------------------------------------+
//|                                               Reverse_Trades.mq5 |
//|                                  Copyright 2024, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//|      Converted to mq5 from ReverseTradesForAllSymbols_Script.mq4 |
//|                                               by: Ryan L Johnson |
//|                                        www.mql5.com/en/users/rjo |
//|                                     mq4 author: Iulian Persinaru |
//|                                   mq4 email: persinaru@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2024, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
#property script_show_inputs

#include <Trade\Trade.mqh>
CTrade trade;

// Input parameters
input bool reverseAllTrades = false;            // Reverse All Trades

long pos_id;
string pos_id_str;
string lastSymbolClosed;
string lastSymbolOpened;

//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
{
 if (reverseAllTrades)
  {
   ReverseAllTrades();
  }
}

//+------------------------------------------------------------------+
//| Reverse all trades function                                      |
//+------------------------------------------------------------------+
void ReverseAllTrades()
{   
	int positions = PositionsTotal();

	for(int i=0; i<positions; i++)
     {
	   ResetLastError();
	   string symbol = PositionGetSymbol(i);
		if(symbol != "")
        {
		   pos_id					   = PositionGetInteger(POSITION_IDENTIFIER);
		   pos_id_str					= (string)pos_id;
		   double price				= PositionGetDouble(POSITION_PRICE_OPEN);
		   double lots             = PositionGetDouble(POSITION_VOLUME);
		   double stoploss			= PositionGetDouble(POSITION_SL);
		   double takeprofit			= PositionGetDouble(POSITION_TP);
		   ENUM_POSITION_TYPE type = (ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE);
		   long pos_magic				= PositionGetInteger(POSITION_MAGIC);
		   string Comment				= PositionGetString(POSITION_COMMENT);	  

            // Close existing order           
            if(PositionsTotal() > 0)
            {
                bool closeResult;
                double closedPrice;
             
                if(type == POSITION_TYPE_BUY && symbol != lastSymbolClosed)
                  {
                   closedPrice = SymbolInfoDouble(symbol, SYMBOL_BID);
                   closeResult = trade.PositionClose(pos_id, 5);
                   if (closeResult)
                      {
                       Print("Closed order: ", pos_id, " for symbol: ", symbol, " at price: ", closedPrice);
                       lastSymbolClosed = symbol;
                      }
                  }
                if(type == POSITION_TYPE_SELL && symbol != lastSymbolClosed)
                  {
                   closedPrice = SymbolInfoDouble(symbol, SYMBOL_ASK);
                   closeResult = trade.PositionClose(pos_id, 5);
                   if (closeResult)
                      {
                       Print("Closed order: ", pos_id, " for symbol: ", symbol, " at price: ", closedPrice);
                       lastSymbolClosed = symbol;
                      }
                  }
                
                // Open opposite order
                double oppositePrice;
                if(type == POSITION_TYPE_BUY && symbol != lastSymbolOpened)
                   {
                    oppositePrice = SymbolInfoDouble(symbol, SYMBOL_BID);
                
                    int ticket = trade.Sell(lots, symbol, oppositePrice, 0, 0, NULL);
                
                    if (ticket > 0)
                        {
                         Print("Opened opposite order: ", ticket, " for symbol: ", symbol, " at price: ", oppositePrice);
                         lastSymbolOpened = symbol;
                        }
                    else
                        {
                         Print("Failed to open opposite order for symbol: ", symbol);
                        }
                   }
                if(type ==  POSITION_TYPE_SELL && symbol != lastSymbolOpened)
                   {
                    oppositePrice = SymbolInfoDouble(symbol, SYMBOL_ASK);
                
                    int ticket = trade.Sell(lots, symbol, oppositePrice, 0, 0, NULL);
                
                    if (ticket > 0)
                        {
                         Print("Opened opposite order: ", ticket, " for symbol: ", symbol, " at price: ", oppositePrice);
                         lastSymbolOpened = symbol;
                        }
                    else
                        {
                         Print("Failed to open opposite order for symbol: ", symbol);
                        }
                   }
             }
             else
                {
                    Print("Failed to close order: ", pos_id, " for symbol: ", symbol);
                }
            // Pause for a short time to avoid rapid execution
            Sleep(1000);
        }
    }
}
